class DropOffType:
    REGULAR_PICKUP = 'REGULAR_PICKUP'
    REQUEST_COURIER = 'REQUEST_COURIER'

class PaymentInfo:
    PAYMENT_CFR = 'CFR'
    PAYMENT_CIF = 'CIF'
    PAYMENT_CIP = 'CIP'
    PAYMENT_CPT = 'CPT'
    PAYMENT_DAF = 'DAF'
    PAYMENT_DDP = 'DPP'
    PAYMENT_DDU = 'DDU'
    PAYMENT_DAP = 'DAP'
    PAYMENT_DEQ = 'DEQ'
    PAYMENT_DES = 'DES'
    PAYMENT_EXW = 'EXW'
    PAYMENT_FAS = 'FAS'
    PAYMENT_FCA = 'FCA'
    PAYMENT_FOB = 'FOB'

class ContentType:
    DOCUMENTS = 'DOCUMENTS'
    NON_DOCUMENTS = 'NON_DOCUMENTS'

class Measurement:
    MEASUREMENT_METRIC = 'SI'
    MEASUREMENT_US = 'SU'

class OnDemandDelivery:
    SERVICE_POINT = 'TV'
    LEAVE_AT_NEIGHBOUR = 'SW'
    SIGNATURE_NEEDED = 'SX'

class RegistrationNumberType:
    VAT = 'VAT'
    EIN = 'EIN'
    GST = 'GST'
    SSN = 'SSN'
    EOR = 'EOR'
    DUN = 'DUN'
    FED = 'FED'
    STA = 'STA'
    CNP = 'CNP'
    IE = 'IE'
    INN = 'INN'
    KPP = 'KPP'
    OGR = 'OGR'
    OKP = 'OKP'
    MRN = 'MRN'
    OSR = 'OSR'

class NotificationMethods:
    EMAIL = 'EMAIL'


EU_COUNTRIES = {
    'BE' : 'Belgium',
    'BG' : 'Bulgaria',
    'CZ' : 'Czechia',
    'DK' : 'Denmark',
    'DE' : 'Germany',
    'EE' : 'Estonia',
    'IE' : 'Ireland',
    'EL' : 'Greece',
    'ES' : 'Spain',
    'FR' : 'France',
    'HR' : 'Croatia',
    'IT' : 'Italy',
    'CY' : 'Cyprus',
    'LV' : 'Latvia',
    'LT' : 'Lithuania',
    'LU' : 'Luxembourg',
    'HU' :' Hungary',
    'MT' : 'Malta',
    'NL' : 'Netherlands',
    'AT' : 'Austria',
    'PL' : 'Poland',
    'PT' : 'Portugal',
    'RO' : 'Romania',
    'SI' : 'Slovenia',
    'FI' : 'Finland',
    'SE': 'Sweden'
}


PRODUCTS_GLOBAL = {
    'K' : 'Express 9:00 - Documents',
    'E' : 'Express 9:00 - NonDocuments (outside EU)',

    'L' : 'Express 10:30 - Documents (USA)',
    'M' : 'Express 10:30 - NonDocuments (USA)',

    'T' : 'Express 12:00 - Documents',
    'Y' : 'Express 12:00 - NonDocuments (outside EU)',

    'U' : 'Express Worldwide (EU)',
    'D' : 'Express Worldwide - Documents (outside EU)',
    'P' : 'Express Worldwide - NonDocuments (outside EU)',

    'I' : 'Express 9:00  (Domestic)',
    '1' : 'Express 12:00 (Domestic)',
    'N' : 'Express 18:00 (Domestic)',

    'W' : 'Economy Select - Documents (EU)',
    'H' : 'Economy Select - NonDocuments (outside EU)',

    'X' : 'Express Enveloppe - Documents under 0.3kg',
}


from .base import ObjectListModel, BaseModel
from .general import Address, ClientDetail, Dimensions, Notification, Request, Billing, DocumentImages, CustomerBarcode, CustomerLogo, Weight, Dimensions

class ShipmentRequest(BaseModel):

    def __init__(self,
        messageId=None,
        clientDetail=None,
        request=None,
        requestedShipment=None
    ):

        self.messageId = messageId
        self.clientDetail = clientDetail if clientDetail else ClientDetail()
        self.request = request if request else Request()
        self.requestedShipment = requestedShipment if requestedShipment else RequestedShipment()

class RequestedShipment(BaseModel):

    def __init__(self,
        shipmentInfo=None,
        shipTimestamp=None,
        pickupLocationCloseTime=None,
        specialPickupInstruction=None,
        pickupLocation=None,
        paymentInfo=None,
        internationalDetail=None,
        onDemandDeliveryOptions=None,
        onDemandDeliveryURLRequest=None,
        ship=None,
        packages=None,
        dangerousGoods=None,
        getRateEstimates=None,
        shipmentNotifications=None
    ):

        self.shipmentInfo = shipmentInfo if shipmentInfo else ShipmentInfo()
        self.shipTimestamp = shipTimestamp
        self.pickupLocationCloseTime = pickupLocationCloseTime
        self.specialPickupInstruction = specialPickupInstruction
        self.pickupLocation = pickupLocation
        self.paymentInfo = paymentInfo
        self.internationalDetail = internationalDetail if internationalDetail else InternationalDetail()
        self.onDemandDeliveryOptions = onDemandDeliveryOptions if onDemandDeliveryOptions else OnDemandDeliveryOptions()
        self.onDemandDeliveryURLRequest = onDemandDeliveryURLRequest
        self.ship = ship if ship else Ship()
        self.packages = packages if packages else ShipmentPackages()
        self.dangerousGoods = dangerousGoods if dangerousGoods else DangerousGoods()
        self.getRateEstimates = getRateEstimates
        self.shipmentNotifications = shipmentNotifications if shipmentNotifications else ShipmentNotifications()

class ShipmentNotifications(ObjectListModel):

    def __init__(self):
        super(ShipmentNotifications, self).__init__(list=[], listObject=ShipmentNotification)

class ShipmentNotification(BaseModel):

    def __init__(self,
        notificationMethod=None,
        emailAddress=None,
        mobilePhoneNumber=None,
        bespokeMessage=None,
        languageCode=None,
        languageCountryCode=None
    ):

        self.notificationMethod = notificationMethod
        self.emailAddress = emailAddress
        self.mobilePhoneNumber = mobilePhoneNumber
        self.bespokeMessage = bespokeMessage
        self.languageCode = languageCode
        self.languageCountryCode = languageCountryCode

class Ship(BaseModel):

    def __init__(self,
        shipper=None,
        pickup=None,
        bookingRequestor=None,
        buyer=None,
        recipient=None
    ):

        self.shipper = shipper if shipper else PersonalInfo()
        self.pickup = pickup if pickup else PersonalInfo()
        self.bookingRequestor = bookingRequestor if bookingRequestor else PersonalInfo()
        self.buyer = buyer if buyer else PersonalInfo()
        self.recipient = recipient if recipient else PersonalInfo()

class PersonalInfo(BaseModel):
    
    def __init__(self,
        contact=None,
        address=None,
        registrationNumbers=None,
        bankDetail=None
    ):

        self.contact = contact if contact else Contact()
        self.address = address if address else Address()
        self.registrationNumbers = registrationNumbers if registrationNumbers else RegistrationNumbers()
        self.bankDetail = bankDetail if bankDetail else BankDetail()

class BankDetail(BaseModel):

    def __init__(self,
        bankName=None
    ):

        self.bankName = bankName

class RegistrationNumbers(ObjectListModel):

    def __init__(self):
        super(RegistrationNumbers, self).__init__(list=[], listObject=RegistrationNumber)

class RegistrationNumber(BaseModel):

    def __init__(self,
        number=None,
        numberTypeCode=None,
        numberIssuerCountryCode=None
    ):

        self.number = number
        self.numberTypeCode = numberTypeCode
        self.numberIssuerCountryCode = numberIssuerCountryCode

class Contact(BaseModel):

    def __init__(self,
        personName=None,
        companyName=None,
        phoneNumber=None,
        emailAddress=None,
        mobilePhoneNumber=None
    ):

        self.personName = personName
        self.companyName = companyName
        self.phoneNumber = phoneNumber
        self.emailAddress = emailAddress
        self.mobilePhoneNumber = mobilePhoneNumber

class OnDemandDeliveryOptions(BaseModel):

    def __init__(self,
        deliveryOption=None,
        location=None,
        instructions=None,
        gateCode=None,
        LWNTypeCode=None,
        neighbourName=None,
        neighbourHouseNumber=None,
        authorizerName=None,
        selectedServicePointID=None,
        requestedDeliveryDate=None
    ):

        self.deliveryOption = deliveryOption
        self.location = location
        self.instructions = instructions
        self.gateCode = gateCode
        self.LWNTypeCode = LWNTypeCode
        self.neighbourName = neighbourName
        self.neighbourHouseNumber = neighbourHouseNumber
        self.authorizerName = authorizerName
        self.selectedServicePointID = selectedServicePointID
        self.requestedDeliveryDate = requestedDeliveryDate

class InternationalDetail(BaseModel):

    def __init__(self,
        commodities=None,
        content=None,
        exportDeclaration=None
    ):

        self.commodities = commodities if commodities else Commodities()
        self.content = content
        self.exportDeclaration = exportDeclaration if exportDeclaration else ExportDeclaration()

class Commodities(BaseModel):

    def __init__(self,
        description=None,
        customsValue=None,
        USFillingTypeValue=None
    ):

        self.description = description
        self.customsValue = customsValue
        self.USFillingTypeValue = USFillingTypeValue


class ExportDeclaration(BaseModel):

    def __init__(self,
        destinationPort=None,
        exporterCode=None,
        exporterID=None,
        exportLicense=None,
        exportLineItems=None,
        exportReason=None,
        importLicense=None,
        invoiceDate=None,
        invoiceNumber=None,
        invoiceDeclarationTexts=None,
        invoiceSignatureDetails=None,
        otherCharges=None,
        packageMarks=None,
        payerGSTVAT=None,
        recipientReference=None,
        remarks=None,
        termsOfPayment=None
    ):

        self.destinationPort = destinationPort
        self.exporterCode = exporterCode
        self.exporterID = exporterID
        self.exportLicense = exportLicense
        self.exportLineItems = exportLineItems if exportLineItems else ExportLineItems()
        self.exportReason = exportReason
        self.importLicense = importLicense
        self.invoiceDate = invoiceDate
        self.invoiceNumber = invoiceNumber
        self.invoiceDeclarationTexts = invoiceDeclarationTexts
        self.invoiceSignatureDetails = invoiceSignatureDetails
        self.otherCharges = otherCharges
        self.packageMarks = packageMarks
        self.payerGSTVAT = payerGSTVAT
        self.recipientReference = recipientReference
        self.remarks = remarks
        self.termsOfPayment = termsOfPayment

class ExportLineItems(BaseModel):
    
    def __init__(self,
        exportLineItem=None
    ):
        
        self.exportLineItem = exportLineItem if exportLineItem else ExportLineItem()

class ExportLineItem(ObjectListModel):
    
    def __init__(self):
        super().__init__(list=[], listObject=ExportLineItemDetail)
    
class ExportLineItemDetail(BaseModel):
    
    def __init__(self,
        grossWeight=None,
        manufacturingCountryCode=None,
        quantityUnitOfMeasurement=None,
        commodityCode=None,
        quantity=None,
        unitPrice=None,
        itemDescription=None,
        itemNumber=None,
        netWeight=None
    ):
        
        self.grossWeight = grossWeight
        self.manufacturingCountryCode = manufacturingCountryCode
        self.quantityUnitOfMeasurement = quantityUnitOfMeasurement
        self.commodityCode = commodityCode
        self.quantity = quantity
        self.unitPrice = unitPrice
        self.itemDescription = itemDescription
        self.itemNumber = itemNumber
        self.netWeight = netWeight

class ShipmentInfo(BaseModel):

    def __init__(self,
        dropOffType=None,
        rateRequestConfirmed=None,
        serviceType=None,
        localServiceType=None,
        account=None,
        billing=None,
        specialServices=None,
        currency=None,
        unitOfMeasurement=None,
        shipmentIdentificationNumber=None,
        useOwnShipmentIdentificationNumber=None,
        packagesCount=None,
        sendPackage=None,
        labelType=None,
        labelTemplate=None,
        archiveLabelTemplate=None,
        customsInvoiceTemplate=None,
        shipmentReceiptTemplate=None,
        paperlessTradeEnabled=None,
        paperlessTradeImage=None,
        documentImages=None,
        labelOptions=None,
        shipmentReferences=None,
        parentShipmentIdentificationNumber=None,
        requestTransliterateResponse=None,
        requestAdditionalInformation=None,
        requestEstimatedDeliveryDate=None,
        estimatedDeliveryDateType=None,
        requestPickupDetails=None
    ):

        self.dropOffType = dropOffType
        self.rateRequestConfirmed = rateRequestConfirmed
        self.serviceType = serviceType
        self.localServiceType = localServiceType
        self.account = account
        self.billing = billing if billing else ShipmentBilling()
        self.specialServices = specialServices if specialServices else SpecialServices()
        self.currency = currency
        self.unitOfMeasurement = unitOfMeasurement
        self.shipmentIdentificationNumber = shipmentIdentificationNumber
        self.useOwnShipmentIdentificationNumber = useOwnShipmentIdentificationNumber
        self.packagesCount = packagesCount
        self.sendPackage = sendPackage
        self.labelType = labelType
        self.labelTemplate = labelTemplate
        self.archiveLabelTemplate = archiveLabelTemplate
        self.customsInvoiceTemplate = customsInvoiceTemplate
        self.shipmentReceiptTemplate = shipmentReceiptTemplate
        self.paperlessTradeEnabled = paperlessTradeEnabled
        self.paperlessTradeImage = paperlessTradeImage
        self.documentImages = documentImages if documentImages else DocumentImages()
        self.labelOptions = labelOptions if labelOptions else LabelOptions()
        self.shipmentReferences = shipmentReferences if shipmentReferences else ShipmentReferences()
        self.parentShipmentIdentificationNumber = parentShipmentIdentificationNumber
        self.requestTransliterateResponse = requestTransliterateResponse
        self.requestAdditionalInformation = requestAdditionalInformation
        self.requestEstimatedDeliveryDate = requestEstimatedDeliveryDate
        self.estimatedDeliveryDateType = estimatedDeliveryDateType
        self.requestPickupDetails = requestPickupDetails

class ShipmentBilling(Billing):

    def __init__(self,
        shipperAccountNumber=None,
        shippingPaymentType=None,
        billingAccountNumber=None,
        dutyAndTaxPayerAccountNumber=None,
        shipmentPrepaidTotalCharge=None,
        neverOverrideBillingService=None
    ):

        super().__init__(shipperAccountNumber, shippingPaymentType, billingAccountNumber)
        self.dutyAndTaxPayerAccountNumber = dutyAndTaxPayerAccountNumber
        self.shipmentPrepaidTotalCharge = shipmentPrepaidTotalCharge if shipmentPrepaidTotalCharge else ShipmentPrepaidTotalCharge()
        self.neverOverrideBillingService = neverOverrideBillingService

class ShipmentPrepaidTotalCharge(BaseModel):

    def __init__(self,
        currencyCode=None,
        amount=None,
        paymentMethod=None
    ):

        self.currencyCode = currencyCode
        self.amount = amount
        self.paymentMethod = paymentMethod

class SpecialServices(ObjectListModel):

    def __init__(self):
        super(SpecialServices, self).__init__(list=[], listObject=SpecialService)

class SpecialService(BaseModel):

    def __init__(self,
        serviceType=None,
        serviceValue=None,
        currencyCode=None,
        paymentCode=None,
        startDate=None,
        endDate=None,
        textInstruction=None
    ):

        self.serviceType = serviceType
        self.serviceValue = serviceValue
        self.currencyCode = currencyCode
        self.paymentCode = paymentCode
        self.startDate = startDate
        self.endDate = endDate
        self.textInstruction = textInstruction


class LabelOptions(BaseModel):
    
    def __init__(self,
        customerLogo=None,
        customerBarcode=None,
        printerDPI=None,
        requestWaybillDocument=None,
        hideAccountInWaybillDocument=None,
        numberOfWaybillDocumentCopies=None,
        requestDHLCustomsInvoice=None,
        DHLCustomsInvoiceLanguageCode=None,
        DHLCustomsInvoiceLanguageCountryCode=None,
        DHLCustomsInvoiceType=None,
        requestShipmentReceipt=None,
        detachOptions=None,
        requestBarcodeInfo=None,
        labelRegText=None,
        custData=None,
        requestLabelsToFitA4=None,
        requestDHLLogoOnLabel=None
    ):

        self.customerLogo = customerLogo if customerLogo else CustomerLogo()
        self.customerBarcode = customerBarcode if customerBarcode else CustomerBarcode()
        self.printerDPI = printerDPI
        self.requestWaybillDocument = requestWaybillDocument
        self.hideAccountInWaybillDocument = hideAccountInWaybillDocument
        self.numberOfWaybillDocumentCopies = numberOfWaybillDocumentCopies
        self.requestDHLCustomsInvoice = requestDHLCustomsInvoice
        self.DHLCustomsInvoiceLanguageCode = DHLCustomsInvoiceLanguageCode
        self.DHLCustomsInvoiceLanguageCountryCode = DHLCustomsInvoiceLanguageCountryCode
        self.DHLCustomsInvoiceType = DHLCustomsInvoiceType
        self.requestShipmentReceipt = requestShipmentReceipt
        self.detachOptions = detachOptions if detachOptions else DetachOptions()
        self.requestBarcodeInfo = requestBarcodeInfo
        self.labelRegText = labelRegText
        self.custData = custData
        self.requestLabelsToFitA4 = requestLabelsToFitA4
        self.requestDHLLogoOnLabel = requestDHLLogoOnLabel

class DetachOptions(BaseModel):

    def __init__(self,
        allInOnePDF=None,
        splitShipmentReceiptAndCustomsInvoice=None,
        splitTransportLabelAndWaybillDocument=None,
        splitLabelsByPieces=None,
        shipmentReceiptWithLabels=None
    ):

        self.allInOnePDF = allInOnePDF
        self.splitShipmentReceiptAndCustomsInvoice = splitShipmentReceiptAndCustomsInvoice
        self.splitTransportLabelAndWaybillDocument = splitTransportLabelAndWaybillDocument
        self.splitLabelsByPieces = splitLabelsByPieces
        self.shipmentReceiptWithLabels = shipmentReceiptWithLabels

class ShipmentReferences(ObjectListModel):

    def __init__(self):
        super(ShipmentReferences, self).__init__(list=[], listObject=ShipmentReference)

class ShipmentReference(BaseModel):

    def __init__(self,
        shipmentReference=None,
        shipmentReferenceType=None
    ):

        self.shipmentReference = shipmentReference
        self.shipmentReferenceType = shipmentReferenceType

class ShipmentPackages(BaseModel):

    def __init__(self,
        requestedPackages=None
    ):

        self.requestedPackages = requestedPackages if requestedPackages else ShipmentPackage()


    def remove(self, item):
        return self.requestedPackages.remove(item)

    def add(self, item):
        return self.requestedPackages.add(item)
    
    def items(self):
        return self.requestedPackages.items()

class ShipmentPackage(ObjectListModel):

    def __init__(self):
        super(ShipmentPackage, self).__init__(list=[], listObject=ShipmentPackageItem)


class ShipmentPackageItem(BaseModel):

    def __init__(self,
        _number=None,
        weight=None,
        dimensions=None,
        pieceIdentificationNumber=None,
        packageContentDescription=None,
        customerReferences=None,
        parentPieceIdentificationNumber=None,
        packageReferences=None
    ):

        self._number = _number
        self.weight = weight
        self.dimensions = dimensions if dimensions else Dimensions()
        self.pieceIdentificationNumber = pieceIdentificationNumber
        self.packageContentDescription = packageContentDescription
        self.customerReferences = customerReferences
        self.parentPieceIdentificationNumber = parentPieceIdentificationNumber
        self.packageReferences = packageReferences if packageReferences else PackageReferences()

class PackageReferences(ObjectListModel):

    def __init__(self):
        super(PackageReferences, self).__init__(list=[], listObject=PackageReference)
    
class PackageReference(BaseModel):

    def __init__(self,
        packageReference=None,
        packageReferenceType=None
    ):

        self.packageReference = packageReference
        self.packageReferenceType = packageReferenceType

class DangerousGoods(ObjectListModel):

    def __init__(self):
        super(DangerousGoods, self).__init__(list=[], listObject=Content)

class Content(BaseModel):

    def __init__(self,
        contentID=None,
        dryIceTotalNetWeight=None,
        UNCode=None
    ):

        self.contentID = contentID
        self.dryIceTotalNetWeight = dryIceTotalNetWeight
        self.UNCode = UNCode


class ShipmentResponse(BaseModel):

    def __init__(self,
        notification=None,
        packagesResult=None,
        labelImage=None,
        documents=None,
        shipmentIdentificationNumber=None,
        dispatchConfirmationNumber=None,
        onDemandDeliveryURL=None,
        totalNet=None,
        additionalInformation=None,
        barcodeInfo=None,
        estimatedDeliveryDate=None,
        pickupDetails=None
    ):

        self.notification = notification if notification else Notification()
        self.packagesResult = packagesResult if packagesResult else PackagesResult()
        self.labelImage = labelImage if labelImage else LabelImage()
        self.documents = documents
        self.shipmentIdentificationNumber = shipmentIdentificationNumber
        self.dispatchConfirmationNumber = dispatchConfirmationNumber
        self.onDemandDeliveryURL = onDemandDeliveryURL
        self.totalNet = totalNet
        self.additionalInformation = additionalInformation
        self.barcodeInfo = barcodeInfo
        self.estimatedDeliveryDate = estimatedDeliveryDate
        self.pickupDetails = pickupDetails

class PackagesResult(BaseModel):

    def __init__(self,
        packageResult=None
    ):

        self.packageResult = packageResult if packageResult else PackageResult()
    
    def remove(self, item):
        return self.packageResult.remove(item)
        
    def add(self, item):
        return self.packageResult.add(item)

    def items(self):
        return self.packageResult.items()

class PackageResult(ObjectListModel):

    def __init__(self):
        super(PackageResult, self).__init__(list=[], listObject=PackageResultItem)

class PackageResultItem(BaseModel):

    def __init__(self,
        number=None,
        trackingNumber=None
    ):

        self.number = number
        self.trackingNumber = trackingNumber

class LabelImage(ObjectListModel):
    def __init__(self):
        super(LabelImage, self).__init__(list=[], listObject=LabelImageItem)

class LabelImageItem(BaseModel):

    def __init__(self,
        labelImageFormat=None,
        graphicImage=None,
        HTMLImage=None,
        labelImageName=None
    ):
    
        self.labelImageFormat = labelImageFormat
        self.graphicImage = graphicImage
        self.HTMLImage = HTMLImage
        self.labelImageName = labelImageName


from .base import APIEndpoint
import datetime

from mydhl.models.shipmentrequests import *
from mydhl.constants.constants import *

class ShipmentRequestMethods(APIEndpoint):

    def __init__(self, api):
        super(ShipmentRequestMethods, self).__init__(api, "ShipmentRequest")
    
    def shipPackage(self,
        serviceType,
        shipper, 
        recipient,
        packages,
        internationalDetail,
        labelOptions=None,
        shipTimestamp=None,
        dropOffType=DropOffType.REGULAR_PICKUP, 
        unitOfMeasurement=Measurement.MEASUREMENT_METRIC,
        paymentInfo=PaymentInfo.PAYMENT_DAP,
        currency='EUR',
        invoicePDF=None,
    ):

        # Create timestamp from now +2 hours if timestamp is not given
        if not shipTimestamp: 
            shipTime = datetime.datetime.now() + datetime.timedelta(hours=2)
            shipTimestamp = shipTime.strftime('%Y-%m-%dT%H:%M:%S')
        
        # Starting data
        data = {
            "ShipmentRequest": {
                "RequestedShipment": {
                    "ShipmentInfo": {
                        "LabelType": "PDF",
                        "DropOffType": dropOffType,
                        "ServiceType": serviceType,
                        "Account": self.api.account,
                        "Currency": currency,
                        "UnitOfMeasurement": unitOfMeasurement,
                    },
                    "ShipTimestamp": "{shipTimestamp} GMT+02:00".format(shipTimestamp=shipTimestamp),
                    "PaymentInfo": paymentInfo,
                    "InternationalDetail" : internationalDetail.getJSON(),
                    "Ship": {
                        "Shipper": shipper.getJSON(),
                        "Recipient": recipient.getJSON()
                    },
                    "Packages": packages.getJSON()
                }
            }
        }
        
        # If labelOptions is added, send it with the request
        if labelOptions: data['ShipmentRequest']['RequestedShipment']['ShipmentInfo']['LabelOptions'] = labelOptions.getJSON()
        
        # If PDF of invoice is added, send it with the request
        if invoicePDF:
            specialServices = SpecialServices()
            serv = SpecialService(serviceType="WY")
            specialServices.add(serv)
            
            # Enable paperless trade
            data['ShipmentRequest']['RequestedShipment']['ShipmentInfo']['PaperlessTradeEnabled'] = 'true'
            
            # Add WY service
            data['ShipmentRequest']['RequestedShipment']['ShipmentInfo']['SpecialServices'] = [{ 'Service' : [] }]
            data['ShipmentRequest']['RequestedShipment']['ShipmentInfo']['SpecialServices'][0]['Service'] = specialServices.getJSON()
            
            # Add image in B64
            data['ShipmentRequest']['RequestedShipment']['ShipmentInfo']['PaperlessTradeImage'] = invoicePDF
            data['ShipmentRequest']['RequestedShipment']['ShipmentInfo']['DocumentImages'] = [{ 'DocumentImage' : { 'DocumentImageType' : 'DCL', 'DocumentImage' : invoicePDF, 'DocumentImageFormat' : 'PDF'}}]

        url = self.endpoint

        status, headers, respJson = self.api.post(url, data)

        print(respJson)
        return ShipmentResponse().parse(respJson['ShipmentResponse'])



#==============[HELPER FUNCTIONS]=================#

def getIndexWithValue(list, attribute, value):

    for index, obj in enumerate(list):
        if hasattr(obj, attribute):
            if getattr(obj, attribute) == value:
                return index
    
    return None

def getObjectWithValue(list, attribute, value):

    for index, obj in enumerate(list):
        if hasattr(obj, attribute):
            if getattr(obj, attribute) == value:
                return obj
    
    return None

def formatKey(string):
    return (string[0].upper() + string[1:]).replace('_', '@')


#==============[BASE MODELS]=================#

class BaseModel:

    def parse(self, json):
        for key, value in json.items():
            lowerKey = ''.join(e.lower() for e in key if e.isalnum())
            lowerAttrs = { k.replace('_', '').lower() : k for k in self.__dict__.keys() }

            if lowerKey in lowerAttrs.keys():
                key = lowerAttrs[lowerKey]
                attrVal = getattr(self, key)

                if isinstance(attrVal, BaseModel):
                    setattr(self, key, attrVal.parse(value))
                else:
                    setattr(self, key, value)

        return self
    
    def getJSON(self):

        dikt = {}
        for k, v in self.__dict__.items():
            if v:
                k = formatKey(k)
                if isinstance(v, BaseModel):
                    json = v.getJSON()
                    if json: dikt[k] = json
                else:
                    dikt[k] = v

        return dikt if len(dikt) > 0 else None

class ObjectListModel(BaseModel):

    def __init__(self, list=[], listObject=None):

        self.list = list
        self.listObject = listObject
    
    def add(self, item):
        self.list.append(item)
        return self.list
    
    def remove(self, item):
        self.list.remove(item)
        return self.list

    def getItemIndex(self, attribute, value):
        index = getIndexWithValue(self.list, attribute, value)
        return index
    
    def getItemObject(self, attribute, value):
        object = getObjectWithValue(self.list, attribute, value)
        return object
    
    def parse(self, json):

        if isinstance(json, dict):
            itemObj = self.listObject().parse(json)
            self.add(itemObj)
        elif isinstance(json, list):
            for item in json:
                itemObj = self.listObject().parse(item)
                self.add(itemObj)

        return self
    
    def getJSON(self):
        list = []

        for item in self.list:
            list.append(item.getJSON())
        
        return list if len(list) > 0 else None

    def items(self):
        return self.list


from .base import APIEndpoint
from datetime import datetime

from mydhl.models.raterequests import *
from mydhl.constants.constants import *

class RateRequestMethods(APIEndpoint):

    def __init__(self, api):
        super(RateRequestMethods, self).__init__(api, "RateRequest")
    

    def get(self, 
        shipper, 
        recipient,
        packages,
        clientDetails=None,
        shipTimestamp=None,
        dropOffType=DropOffType.REGULAR_PICKUP, 
        unitOfMeasurement=Measurement.MEASUREMENT_METRIC, 
        content=ContentType.DOCUMENTS,
        paymentInfo=PaymentInfo.PAYMENT_DAP,
        nextBusinessDay='N'
    ):

        if not shipTimestamp: shipTimestamp = datetime.now().strftime('%Y-%m-%dT%H:%M:%S')

        data = {
            "RateRequest" : {
                'ClientDetails' : clientDetails,
                "RequestedShipment" : {
                    "DropOffType" : dropOffType,
                    "ShipTimestamp" : "{shipTimestamp} GMT+02:00".format(shipTimestamp=shipTimestamp),
                    "UnitOfMeasurement" : unitOfMeasurement,
                    "Content" : content,
                    "PaymentInfo" : paymentInfo,
                    "NextBusinessDay" : nextBusinessDay,
                    "Account" : self.api.account,
                    'Ship' : {
                        'Shipper' : shipper.getJSON(),
                        'Recipient' : recipient.getJSON()
                    },
                    'Packages' : packages.getJSON()
                }
            }
        }

        url = self.endpoint

        status, headers, respJson = self.api.post(url, data)
        return RateResponse().parse(respJson['RateResponse'])


from mydhl.models.shipmentrequests import Address
from .base import ObjectListModel, BaseModel
from .general import ClientDetail, Request, Address, Notification, NotificationItem
from mydhl.constants.constants import *

class RateResponse(BaseModel):

    def __init__(self, 
        provider=None
    ):

        self.provider = provider if provider else Provider()
    

class Provider(ObjectListModel):

    def __init__(self):

        super(Provider, self).__init__(list=[], listObject=ProviderItem)

    
class ProviderItem(BaseModel):

    def __init__(self,
        code=None,
        notification=None,
        service=None
    ):

        self.code = code
        self.notification = notification if notification else Notification()
        self.service = service if service else Service()

class Service(ObjectListModel):

    def __init__(self):

        super(Service, self).__init__(list=[], listObject=ServiceItem)
    
class ServiceItem(BaseModel):

    def __init__(self,
        type=None,
        totalNet=None,
        charges=None,
        deliveryTime=None,
        cutOffTime=None,
        nextBusinessDayInd=None
    ):

        self.type = type
        self.totalNet = totalNet if totalNet else TotalNet()
        self.charges = charges if charges else Charges()
        self.deliveryTime = deliveryTime
        self.cutOffTime = cutOffTime
        self.nextBusinessDayInd = nextBusinessDayInd
    
    def getTypeDisplay(self):

        if self.type:
            if self.type in PRODUCTS_GLOBAL: return PRODUCTS_GLOBAL[self.type]
    
        return 'TYPE_NOTFOUND'


class TotalNet(BaseModel):
    
    def __init__(self,
        currency=None,
        amount=None
    ):

        self.currency = currency
        self.amount = amount

class Charges(BaseModel):

    def __init__(self,
        currency=None,
        charge=None
    ):

        self.currency = currency
        self.charge = charge if charge else ChargeList()


class ChargeList(ObjectListModel):

    def __init__(self):

        super(ChargeList, self).__init__(list=[], listObject=Charge)


class Charge(BaseModel):

    def __init__(self,
        chargeCode=None,
        chargeType=None,
        chargeAmount=None
    ):

        self.chargeCode = chargeCode
        self.chargeType = chargeType
        self.chargeAmount = chargeAmount


class RateRequest(BaseModel):

    def __init__(self,
        clientDetail=None,
        request=None,
        requestedShipment=None
    ):

        self.clientDetail = clientDetail if clientDetail else ClientDetail()
        self.request = request if request else Request()
        self.requestedShipment = requestedShipment if requestedShipment else RequestedShipment()

class RequestedShipment(BaseModel):

    def __init__(self,
        dropOffType=None,
        shipTimestamp=None,
        unitOfMeasurement=None,
        content=None,
        paymentInfo=None,
        nextBusinessDay=None,
        account=None,
        ship=None,
        packages=None
    ):

        self.dropOffType = dropOffType
        self.shipTimestamp = shipTimestamp
        self.unitOfMeasurement = unitOfMeasurement
        self.content = content
        self.paymentInfo = paymentInfo
        self.nextBusinessDay = nextBusinessDay
        self.account = account
        self.ship = ship if ship else Ship()
        self.packages = packages if packages else Packages()

class Ship(BaseModel):

    def __init__(self,
        shipper=None,
        recipient=None
    ):

        self.shipper = shipper if shipper else Address()
        self.recipient = recipient if recipient else Address()

class Packages(BaseModel):

    def __init__(self,
        requestedPackages=None
    ):
        self.requestedPackages = requestedPackages if requestedPackages else RequestedPackages()

    def items(self):
        return self.requestedPackages.items()

    def remove(self, item):
        return self.requestedPackages.remove(item)
        
    def add(self, item):
        return self.requestedPackages.add(item)

class RequestedPackages(ObjectListModel):

    def __init__(self):
        super(RequestedPackages, self).__init__(list=[], listObject=RequestedPackage)
    

class RequestedPackage(BaseModel):

    def __init__(self,
        _number=None,
        weight=None,
        dimensions=None
    ):
        self._number = _number
        self.weight = weight if weight else Weight()
        self.dimensions = dimensions if dimensions else Dimensions()


import base64
import requests
import json
import time

from . import config

from .endpoints.raterequests import RateRequestMethods
from .endpoints.shipmentrequests import ShipmentRequestMethods

class MyDHLAPI:

    def __init__(self, account, username, password):

        self.account = account
        self.username = username
        self.password = password

        self.headers = {
            'Content-Type' : 'application/json',
            'Accept' : 'application/json'
        }

        self.baseUrl = config.BASE_URL

        self.rateRequests = RateRequestMethods(self)
        self.shipmentRequests = ShipmentRequestMethods(self)
    
    def setTokenHeader(self, token):
        basicStr = 'Basic {token}'.format(token=token)
        self.headers.update({'Authorization' : basicStr})
    
    def acquireBasicToken(self):
        credentialsString = '{username}:{password}'.format(username=self.username, password=self.password)
        encodedBytes = base64.b64encode(credentialsString.encode('utf-8'))
        encodedToken = str(encodedBytes, 'utf-8')

        self.setTokenHeader(encodedToken)
    
    def checkHeaderTokens(self):
        if 'Authorization' not in self.headers: self.acquireBasicToken()

    def doRequest(self, method, url, data=None, headers=None):

        if headers:
            mergedHeaders = self.headers
            mergedHeaders.update(headers)
            headers = mergedHeaders
        else: headers = self.headers

        reqUrl = '{base}/{url}'.format(base=self.baseUrl, url=url)

        if method == 'GET':
            response = requests.get(reqUrl, params=data, headers=headers)
        elif method == 'POST':
            response = requests.post(reqUrl, data=json.dumps(data), headers=headers)
        elif method == 'PUT':
            response = requests.put(reqUrl, data=json.dumps(data), headers=headers)
        
        return response

    def request(self, method, url, data=None, headers=None):

        self.checkHeaderTokens()
        response = self.doRequest(method, url, data, headers)

        if 'json' in response.headers['Content-Type']:
            respContent = response.json()
        elif 'pdf' in response.headers['Content-Type']:
            respContent = response.content
        
        return response.status_code, response.headers, respContent
    
    def get(self, url, data=None, headers=None):
        status, headers, response = self.request('GET', url, data, headers)
        return status, headers, response
    
    def post(self, url, data=None, headers=None):
        status, headers, response = self.request('POST', url, data, headers)
        return status, headers, response
    
    def put(self, url, data=None, headers=None):
        status, headers, response = self.request('PUT', url, data, headers)
        return status, headers, response


from setuptools import setup

# read the contents of README file
from os import path
this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
  name = 'python-mydhl-api',         
  packages=['mydhl', 'mydhl.models', 'mydhl.constants', 'mydhl.cache', 'mydhl.endpoints'],
  version = '0.2.1',
  license='GPL-3.0-or-later',
  description = 'Wrapper for the MyDHL API (v2.5)',
  long_description=long_description,
  long_description_content_type='text/markdown',
  author = 'Alexander Schillemans',
  author_email = 'alexander.schillemans@lhs.global',
  url = 'https://github.com/alexanderlhsglobal/python-mydhl-api',
  download_url = 'https://github.com/alexanderlhsglobal/python-mydhl-api/archive/refs/tags/0.2.1.tar.gz',
  keywords = ['mydhl', 'api', 'dhl express', 'dhl'],
  install_requires=[
          'requests',
          'python-dateutil',
      ],
  classifiers=[
    'Development Status :: 3 - Alpha',
    'Intended Audience :: Developers',
    'Topic :: Software Development :: Build Tools',
    'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
    'Programming Language :: Python :: 3.6',
  ],
)


from .base import ObjectListModel, BaseModel

class ClientDetail(BaseModel):

    def __init__(self,
        so=None,
        plant=None
    ):

        self.so = so
        self.plant = plant

class Request(BaseModel):

    def __init__(self,
        serviceheader=None
    ):

        self.serviceheader = serviceheader if serviceheader else ServiceHeader()

class ServiceHeader(BaseModel):

    def __init__(self,
        messagetime=None,
        messagereference=None,
        webstoreplatform=None,
        webstoreplatformversion=None,
        shippingsystemplatform=None,
        shippingsystemplatformversion=None,
        plugin=None,
        pluginversion=None
    ):

        self.messagetime = messagetime
        self.messagereference = messagereference
        self.webstoreplatform = webstoreplatform
        self.webstoreplatformversion = webstoreplatformversion
        self.shippingsystemplatform = shippingsystemplatform
        self.shippingsystemplatformversion = shippingsystemplatformversion
        self.plugin = plugin
        self.pluginversion = pluginversion

class Billing(BaseModel):

    def __init__(self,
        shipperAccountNumber=None,
        shippingPaymentType=None,
        billingAccountNumber=None,
    ):

        self.shipperAccountNumber = shipperAccountNumber
        self.shippingPaymentType = shippingPaymentType
        self.billingAccountNumber = billingAccountNumber
    
class DocumentImages(ObjectListModel):
    def __init__(self):
        super(DocumentImages, self).__init__(list=[], listObject=DocumentImage)

class DocumentImage(BaseModel):

    def __init__(self,
        documentImageType=None,
        documentImage=None,
        documentImageFormat=None
    ):

        self.documentImageType = documentImageType
        self.documentImage = documentImage
        self.documentImageFormat = documentImageFormat

class CustomerLogo(BaseModel):

    def __init__(self,
        logoImage=None,
        logoImageFormat=None
    ):

        self.logoImage = logoImage
        self.logoImageFormat = logoImageFormat

class CustomerBarcode(BaseModel):

    def __init__(self,
        barcodeType=None,
        barcodeContent=None,
        textBelowBarcode=None
    ):

        self.barcodeType = barcodeType
        self.barcodeContent = barcodeContent
        self.textBelowBarcode = textBelowBarcode

class Address(BaseModel):

    def __init__(self,
        streetLines=None,
        streetLines2=None,
        streetLines3=None,
        streetName=None,
        streetNumber=None,
        city=None,
        cityDistrict=None,
        stateOrProvinceCode=None,
        postalCode=None,
        countryCode=None
    ):

        self.streetLines = streetLines
        self.streetLines2 = streetLines2
        self.streetLines3 = streetLines3
        self.streetName = streetName
        self.streetNumber = streetNumber
        self.city = city
        self.cityDistrict = cityDistrict
        self.stateOrProvinceCode = stateOrProvinceCode
        self.postalCode = postalCode
        self.countryCode = countryCode


class Weight(BaseModel):

    def __init__(self,
        value=None
    ):

        self.value = value

class Dimensions(BaseModel):

    def __init__(self,
        length=None,
        width=None,
        height=None
    ):

        self.length = length
        self.width = width
        self.height = height

class Notification(ObjectListModel):

    def __init__(self):

        super(Notification, self).__init__(list=[], listObject=NotificationItem)
    

class NotificationItem(BaseModel):
    def __init__(self,
        code=None,
        message=None
    ):

        self.code = code
        self.message = message
